package org.jeecg.ctop.material.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.DateUtils;
import org.jeecg.ctop.material.constants.Check;
import org.jeecg.ctop.material.entity.vo.KuaiShouOperateGroupVo;
import org.jeecg.ctop.material.mapper.KuaishouAccountReportDailyDwMapper;
import org.jeecg.ctop.material.mapper.MaterialStareMapper;
import org.jeecg.ctop.material.service.IKuaishouAccountReportDailyDwService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

@Service
public class KuaishouAccountReportDailyDwServiceImpl implements IKuaishouAccountReportDailyDwService {
    @Resource
    private KuaishouAccountReportDailyDwMapper kuaishouAccountReportDailyDwMapper;

    @Resource
    private MaterialStareMapper materialStareMapper;

    @Override
    public JSONObject getTotalCharge(Map<String, Object> paramsMap) {
        JSONObject ks = kuaishouAccountReportDailyDwMapper.getTotalCharge(paramsMap);
        Long count = kuaishouAccountReportDailyDwMapper.getYesterdayUploadNewMaterialCount(paramsMap);
        ks.put("kuaishouCount", count);
        return ks;
    }

    @Override
    public JSONObject getTrendCharge(String type, Long projectId, String startTime, String endTime) {
        Integer startTimeInt = DateUtils.getDateInteger(startTime);
        Integer endTimeInt = DateUtils.getDateInteger(endTime);

        List<JSONObject> today = null;
        List<JSONObject> yesterday = null;
        if ("1".equals(type)) {
            today = kuaishouAccountReportDailyDwMapper.getProjectNoByDays(startTimeInt, endTimeInt);
        } else if ("2".equals(type)) {
            today = kuaishouAccountReportDailyDwMapper.getAdvertiserNoByDays(startTimeInt, endTimeInt);
        } else if ("3".equals(type)) {
            if (startTime.equals(endTime)) {
                today = kuaishouAccountReportDailyDwMapper.getUnitNoByHours(startTimeInt);
            } else {
                today = kuaishouAccountReportDailyDwMapper.getUnitNoByDays(startTimeInt, endTimeInt);
            }
        } else {
            if (startTime.equals(endTime)) {
                today = kuaishouAccountReportDailyDwMapper.getHourcharge(projectId, startTimeInt);
                yesterday = kuaishouAccountReportDailyDwMapper.getHourcharge(projectId, DateUtils.getDateInteger(DateUtils.getLastDay(startTime, 1)));
            } else {
                today = kuaishouAccountReportDailyDwMapper.getDatacharge(projectId, startTimeInt, endTimeInt);
                fillingDates(startTime, endTime, today);
                int days = DateUtils.dateDiff(startTime, endTime);
                String endTime2 = DateUtils.getLastDay(startTime, 1);
                String startTime2 = DateUtils.getLastDay(endTime2, days);
                yesterday = kuaishouAccountReportDailyDwMapper.getDatacharge(projectId, DateUtils.getDateInteger(startTime2), DateUtils.getDateInteger(endTime2));
                fillingDates(startTime2, endTime2, yesterday);
            }
        }
        JSONObject data = new JSONObject();
        data.put("today", today);
        data.put("yesterday", yesterday);
        return data;
    }

    /**
     * 补充日期
     */
    private void fillingDates(String startTime, String endTime, List<JSONObject> today) {
        long dayNum = DateUtils.getDiscrepantDays(startTime, endTime) + 1;
        int size = today.size();
        if (dayNum - size != 0) {
            List<String> days = DateUtils.getDays(startTime, endTime);
            for (String day : days) {
                if (!today.contains(day)) {
                    JSONObject obj = new JSONObject();
                    obj.put("time", day);
                    obj.put("charge", 0.00);
                    today.add(obj);
                }
            }
        }
    }

    @Override
    public Result<Object> getProjectReport(String category, Long projectId, String startTime, String endTime, String sortCode, String order, Integer pageSize, Integer pageNo) throws Exception {
        if (Check.isNull(order)) {
            order = "DESC";
        }
        if (Check.isNull(sortCode)) {
            sortCode = "charge";
        }

        Integer startTimeInt = DateUtils.getDateInteger(startTime);
        Integer endTimeInt = DateUtils.getDateInteger(endTime);
        //当前小时
        Integer nowHour = DateUtils.getNowHour();
        //当天日期
        String today = DateUtils.formatDate(new Date());
        List<JSONObject> data = null;
        Long total = null;
        if (startTime.equals(endTime) && today.equals(startTime)) {
//            total = kuaishouAccountReportDailyDwMapper.getProjectToDayReportTotal(projectId, startTimeInt, nowHour);
//            PageHelper.startPage(pageNo, pageSize, false);
            data = kuaishouAccountReportDailyDwMapper.getProjectToDayReport(category, projectId, startTimeInt, nowHour, sortCode, order);
        } else {
//            total = kuaishouAccountReportDailyDwMapper.getProjectMoreDaysReportTotal(projectId, startTimeInt, endTimeInt, nowHour);
//            PageHelper.startPage(pageNo, pageSize, false);

            int days = DateUtils.dateDiff(startTime, endTime) + 1;
            //上阶段时间
            Integer lastStart = DateUtils.getDateInteger(DateUtils.getLastDay(startTime, days));
            Integer lastEnd = DateUtils.getDateInteger(DateUtils.getLastDay(endTime, days));
            data = kuaishouAccountReportDailyDwMapper.getProjectMoreDaysReport(projectId, startTimeInt, endTimeInt, lastStart, lastEnd, sortCode, order);
        }
        PageInfo<JSONObject> page = new PageInfo<>();
        page.setList(data);
//        page.setTotal(total);
        return Result.OK(page);
    }

    @Override
    public Result<Object> getOperateInfo(String startTime, String endTime) {
        try {
            //当前小时
            Integer hour = DateUtils.getNowHour();
            //当天日期
            String nowDateStr = DateUtils.formatDate(new Date());
            Long nowDate = Long.valueOf(DateUtils.getDateInteger(nowDateStr));

            Long startDate = Long.valueOf(DateUtils.getDateInteger(startTime));
            Long entDate = Long.valueOf(DateUtils.getDateInteger(endTime));
            //开始日期前一天
            String lastDay = DateUtils.getLastDay(startTime, 1);
            //相差天数，同一天为 0
            int days = DateUtils.dateDiff(startTime, endTime);
            //上阶段时间
            Long lastTimeEnd = Long.valueOf(DateUtils.getDateInteger(lastDay));
            Long lastTimeStart = Long.valueOf(DateUtils.getDateInteger(DateUtils.getLastDay(lastDay, days)));

            List<String> userIds = materialStareMapper.getAllOperationByMediaId("2");
            List<JSONObject> list = new ArrayList<>();
            if (startTime.equals(endTime) && startTime.equals(nowDateStr)) {
                list = kuaishouAccountReportDailyDwMapper.getKuaishouOperateInfoHourly(hour, startDate, entDate, lastTimeEnd, lastTimeStart, userIds);
            } else {
                list = kuaishouAccountReportDailyDwMapper.getKuaishouOperateInfo(startDate, entDate, lastTimeEnd, lastTimeStart, userIds);
            }
            PageInfo<JSONObject> pageInfo = new PageInfo<>(list);
            return Result.successMsg("查询成功。", pageInfo);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.errorMsg("查询失败了。");
    }

    @Override
    public Result<Object> getOperateGroupInfo(String startTime, String endTime) {
        try {
            //当前小时
            Integer hour = DateUtils.getNowHour();
            //当天日期
            String nowDateStr = DateUtils.formatDate(new Date());
            Long nowDate = Long.valueOf(DateUtils.getDateInteger(nowDateStr));

            Long startDate = Long.valueOf(DateUtils.getDateInteger(startTime));
            Long entDate = Long.valueOf(DateUtils.getDateInteger(endTime));

            //开始日期前一天
            String lastDay = DateUtils.getLastDay(startTime, 1);
            //相差天数，同一天为 0
            int days = DateUtils.dateDiff(startTime, endTime);
            //上阶段时间
            Long lastTimeEnd = Long.valueOf(DateUtils.getDateInteger(lastDay));
            Long lastTimeStart = Long.valueOf(DateUtils.getDateInteger(DateUtils.getLastDay(lastDay, days)));

            List<KuaiShouOperateGroupVo> operationManagerList = materialStareMapper.getOperationManagerUser("kuaishouOperationManager");
            List<JSONObject> list = new ArrayList<>();
            for (KuaiShouOperateGroupVo groupVo : operationManagerList) {
                //下级员工
                Set<String> operatorUserIds = materialStareMapper.queryKuaishouOperateTeamMember(groupVo.getUserId(), startDate, entDate);
                if (Check.isNull(operatorUserIds)) {
                    continue;
                }
                //运营数量
                int operationNum = operatorUserIds.size();
                //查询账户数据
                JSONObject vo = new JSONObject();
                if (startTime.equals(endTime) && startTime.equals(nowDateStr)) {
                    //时报
                    vo = kuaishouAccountReportDailyDwMapper.getKuaishouOperateGroupInfoHourly(hour, startDate, entDate, lastTimeEnd, lastTimeStart, operatorUserIds);
                } else {
                    vo = kuaishouAccountReportDailyDwMapper.getKuaishouOperateGroupInfo(startDate, entDate, lastTimeEnd, lastTimeStart, operatorUserIds);
                }
                vo.put("userId", groupVo.getUserId());
                vo.put("realname", groupVo.getRealname());
                vo.put("operationNum", operationNum);
                //总花费
                Double totalCost = vo.getDouble("totalCost");
                //日均消耗 总花费/天数
                Double aveDay = totalCost / (days + 1);
                vo.put("averageDayCost", new BigDecimal(aveDay).setScale(3, BigDecimal.ROUND_HALF_UP).doubleValue());
                //人均消耗
                Double avePeople = totalCost / operationNum;
                vo.put("averagePeopleCost", new BigDecimal(avePeople).setScale(3, BigDecimal.ROUND_HALF_UP).doubleValue());
                //账户数量
                Long accountNum = vo.getLong("accountNum");
                //平均账户消耗 总花费/账户数量
                Double aveAccount = accountNum == 0 ? 0 : totalCost / accountNum;
                vo.put("averageAccountCost", new BigDecimal(aveAccount).setScale(3, BigDecimal.ROUND_HALF_UP).doubleValue());
                list.add(vo);
            }
            if (!Check.isNull(list)) {
                Collections.sort(list, new Comparator<JSONObject>() {
                    @Override
                    public int compare(JSONObject o1, JSONObject o2) {
                        return o2.getDouble("totalCost").compareTo(o1.getDouble("totalCost"));
                    }
                });
            }
            PageInfo<JSONObject> pageInfo = new PageInfo<>(list);
            return Result.successMsg("查询成功。", pageInfo);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.errorMsg("查询失败了。");
    }
}
