package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.constant.CtopRoleCodeConstant;
import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.ai.entity.AiKuaishouAccountAutoStrategy;
import cn.com.ctop.kuaishou.modules.ai.entity.AiKuaishouAdvertiserStrategy;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouAccountAutoStrategyService;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouAdvertiserStrategyService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouVideoRelateCreativesService;
import cn.com.ctop.kuaishou.modules.document.service.DocumentLibraryService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.RedisUtil;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.system.service.ISysRoleService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 自动投放重构-账户策略表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-10-12
 */
@Slf4j
@Api(tags = "自动投放重构-账户策略表")
@RestController
@RequestMapping("/auto/aiKuaishouAccountAutoStrategy")
public class AiKuaishouAccountAutoStrategyController {
    @Autowired
    private IAiKuaishouAccountAutoStrategyService aiKuaishouAccountAutoStrategyService;

    @Autowired
    private ISysRoleService sysRoleService;

    @Autowired
    private IUserAllocationService allocationService;

    @Autowired
    private IKuaishouVideoRelateCreativesService kuaishouVideoRelateCreativesService;

    @Autowired
    private DocumentLibraryService documentLibraryService;

    @Autowired
    private IAiKuaishouAdvertiserStrategyService aiKuaishouAdvertiserStrategyService;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 分页列表查询
     *
     * @param AiKuaishouAccountAutoStrategy
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<PageInfo<AiKuaishouAccountAutoStrategy>> queryPageList(AiKuaishouAccountAutoStrategy aiKuaishouAccountAutoStrategy,
                                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                         HttpServletRequest req) {
        Result<PageInfo<AiKuaishouAccountAutoStrategy>> result = new Result<>();
        if (Check.isNull(aiKuaishouAccountAutoStrategy.getUserId())) {
            result.setSuccess(false);
            result.setMessage("请选择用户");
            return result;
        }
        String userId = aiKuaishouAccountAutoStrategy.getUserId();
        String roleCode = sysRoleService.getRoleCodeByUserId(userId);
        List<String> userIds = new ArrayList<>();
        if ("CompanyManager".equals(roleCode)) {
            userIds = sysRoleService.getCompanyUsers(roleCode, userId);
            aiKuaishouAccountAutoStrategy.setUserIds(userIds);
        } else if ("bytedanceDirector".equals(roleCode)) {
            userIds = sysRoleService.getCompanyUsers(roleCode, userId);
            aiKuaishouAccountAutoStrategy.setUserIds(userIds);
        } else if ("kuaishouDirector".equals(roleCode)) {
            userIds = sysRoleService.getCompanyUsers(roleCode, userId);
            aiKuaishouAccountAutoStrategy.setUserIds(userIds);
        } else if (CtopRoleCodeConstant.COMMON_ROLE_CODE_ADMIN.equals(roleCode)) {
            aiKuaishouAccountAutoStrategy.setUserIds(null);
        } else {
            userIds.add(userId);
            aiKuaishouAccountAutoStrategy.setUserIds(userIds);
        }
        PageHelper.startPage(pageNo, pageSize);
        PageInfo<AiKuaishouAccountAutoStrategy> pageInfo = aiKuaishouAccountAutoStrategyService.queryList(aiKuaishouAccountAutoStrategy);
        List<AiKuaishouAccountAutoStrategy> list = pageInfo.getList();
        List<AiKuaishouAccountAutoStrategy> setData = new ArrayList<>();
        String date = DateUtils.formatDate(new Date());
        if (null != list && !list.isEmpty()) {
            list.forEach(record -> {
                Long accountId = record.getAccountId();
                UserAllocation allocation = allocationService.getByAccountId(accountId);
                if (null != allocation) {
                    record.setAuthName(allocation.getAuthName());
                }
                String customkey = date + "_auto_custom_cnt_" + accountId;
                Integer customUnitCnt = (Integer) redisUtil.get(customkey);
                customUnitCnt = Check.isNull(customUnitCnt) ? 0 : customUnitCnt;
                record.setCustomCreatedUnitCnt(customUnitCnt);
                String programkey = date + "_auto_program_cnt_" + accountId;
                Integer programUnitCnt = (Integer) redisUtil.get(programkey);
                programUnitCnt = Check.isNull(programUnitCnt) ? 0 : programUnitCnt;
                record.setProgramCreatedUnitCnt(programUnitCnt);
            });
        }
        pageInfo.setList(list);
        result.setSuccess(true);
        result.setResult(pageInfo);
        return result;
    }

    /**
     * 校验新旧版本自动投放账户配置不重复
     * type 0 旧版 ，1 重构版
     */
    @GetMapping(value = "/checkStrategy")
    private Result<Object> checkStrategy(Long accountId, Integer type) {
        if (Check.isNull(accountId) || Check.isNull(type)) {
            return Result.error("缺少参数");
        }
        if (type == 1) {
            AiKuaishouAdvertiserStrategy getEntity = aiKuaishouAdvertiserStrategyService.getByAccountId(accountId);
            if (null != getEntity && getEntity.getStatus() == 1) {
                return Result.error("该账户旧版自动投放策略已启动，请关闭后再创建");
            }

        } else {
            QueryWrapper<AiKuaishouAccountAutoStrategy> queryWrapper = new QueryWrapper();
            queryWrapper.eq("account_id", accountId).last("limit 1");
            AiKuaishouAccountAutoStrategy one = aiKuaishouAccountAutoStrategyService.getOne(queryWrapper);
            if (null != one && one.getStatus() == 1) {
                return Result.error("该账户重构版自动投放策略已启动，请关闭后再创建");
            }

        }
        return Result.ok("success");
    }

    /**
     * 添加
     *
     * @param aiKuaishouAccountAutoStrategy
     * @return
     */
    @ApiOperation(value = "自动投放重构-账户策略表-添加", notes = "自动投放重构-账户策略表-添加")
    @PostMapping(value = "/add")
    public Result<AiKuaishouAccountAutoStrategy> add(@RequestBody AiKuaishouAccountAutoStrategy autoStrategy) {
        Result<AiKuaishouAccountAutoStrategy> result = new Result<>();
        try {
            Long accountId = autoStrategy.getAccountId();

            QueryWrapper<AiKuaishouAccountAutoStrategy> queryWrapper = new QueryWrapper();
            queryWrapper.eq("account_id", accountId).last("limit 1");
            AiKuaishouAccountAutoStrategy one = aiKuaishouAccountAutoStrategyService.getOne(queryWrapper);
            if (null != one) {
                result.error500("该账户配置信息已经被创建");
                return result;
            }
            UserAllocation allocation = allocationService.getByAccountId(accountId);
            if (null != allocation) {
                autoStrategy.setUserId(allocation.getUserId());
            }
            Integer singleAppId = autoStrategy.getSingleAppid();
            if (null != singleAppId && singleAppId == 1) {
                if (autoStrategy.getCampaignType() == 2 || autoStrategy.getCampaignType() == 7) {
                    Long appId = Long.valueOf(JSONArray.parseArray(autoStrategy.getAppIdArray()).getString(0));
                    autoStrategy.setAppId(appId);
                }
            }
            aiKuaishouAccountAutoStrategyService.save(autoStrategy);
            //添加广告语到文案库
            documentLibraryService.insertBatch(autoStrategy.getDescription(), accountId);
            Thread thread = new Thread() {
                @Override
                public void run() {
                    kuaishouVideoRelateCreativesService.videoRelateCreatives(autoStrategy.getAccountId());
                }
            };
            thread.start();
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param aiKuaishouAccountAutoStrategy
     * @return
     */
    @ApiOperation(value = "自动投放重构-账户策略表-编辑", notes = "自动投放重构-账户策略表-编辑")
    @PostMapping(value = "/edit")
    public Result<AiKuaishouAccountAutoStrategy> edit(@RequestBody AiKuaishouAccountAutoStrategy aiKuaishouAccountAutoStrategy) {
        Result<AiKuaishouAccountAutoStrategy> result = new Result<AiKuaishouAccountAutoStrategy>();
        AiKuaishouAccountAutoStrategy aiKuaishouAccountAutoStrategyEntity = aiKuaishouAccountAutoStrategyService.getById(aiKuaishouAccountAutoStrategy.getId());
        if (aiKuaishouAccountAutoStrategyEntity == null) {
            result.error500("未找到对应实体");
        } else {
            Integer singleAppId = aiKuaishouAccountAutoStrategy.getSingleAppid();
            if (aiKuaishouAccountAutoStrategyEntity.getCampaignType() == 2 || aiKuaishouAccountAutoStrategyEntity.getCampaignType() == 7) {
                if (null != singleAppId && singleAppId == 1) {
                    Long appId = Long.valueOf(JSONArray.parseArray(aiKuaishouAccountAutoStrategy.getAppIdArray()).getString(0));
                    aiKuaishouAccountAutoStrategy.setAppId(appId);
                }
            }
            if (!Check.isNull(aiKuaishouAccountAutoStrategyEntity.getPlayableIds()) && !aiKuaishouAccountAutoStrategyEntity.getPlayableIds().equals(aiKuaishouAccountAutoStrategy.getPlayableIds())) {
                redisUtil.removeAll(aiKuaishouAccountAutoStrategy.getId() + "-play-");
            }


            boolean ok = aiKuaishouAccountAutoStrategyService.updateById(aiKuaishouAccountAutoStrategy);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 编辑
     *
     * @param aiKuaishouAccountAutoStrategy
     * @return
     */
    @PostMapping(value = "/updateAppUrl")
    public Result<AiKuaishouAccountAutoStrategy> updateAppUrl(@RequestBody AiKuaishouAccountAutoStrategy strategy) {
        Result<AiKuaishouAccountAutoStrategy> result = new Result<AiKuaishouAccountAutoStrategy>();
        if (Check.isNull(strategy.getAccountId())) {
            result.error500("缺少账户数据");
            return result;
        }
        QueryWrapper<AiKuaishouAccountAutoStrategy> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("account_id", strategy.getAccountId()).last("limit 1");
        AiKuaishouAccountAutoStrategy one = aiKuaishouAccountAutoStrategyService.getOne(queryWrapper);
        if (one == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = aiKuaishouAccountAutoStrategyService.update(strategy, queryWrapper);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }


    /**
     * 修改状态
     *
     * @return
     */
    @PostMapping(value = "/updateState")
    public Result<AiKuaishouAccountAutoStrategy> updateState(@RequestBody AiKuaishouAccountAutoStrategy strategy) {
        Result<AiKuaishouAccountAutoStrategy> result = new Result<AiKuaishouAccountAutoStrategy>();
        AiKuaishouAccountAutoStrategy entity = aiKuaishouAccountAutoStrategyService.getById(strategy.getId());
        if (entity == null) {
            result.error500("未找到对应实体");
        } else {
            AiKuaishouAdvertiserStrategy getEntity = aiKuaishouAdvertiserStrategyService.getByAccountId(entity.getAccountId());
            if (null != getEntity && getEntity.getStatus() == strategy.getStatus() && strategy.getStatus() == 1) {
                result.error500("修改失败，该账户旧版自动投放策略已运行");
            } else {
                boolean ok = aiKuaishouAccountAutoStrategyService.updateById(strategy);
                if (ok) {
                    result.success("修改成功!");
                }
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放重构-账户策略表-通过id删除", notes = "自动投放重构-账户策略表-通过id删除")
    @GetMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            aiKuaishouAccountAutoStrategyService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "自动投放重构-账户策略表-批量删除", notes = "自动投放重构-账户策略表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<AiKuaishouAccountAutoStrategy> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<AiKuaishouAccountAutoStrategy> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.aiKuaishouAccountAutoStrategyService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放重构-账户策略表-通过id查询", notes = "自动投放重构-账户策略表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<AiKuaishouAccountAutoStrategy> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<AiKuaishouAccountAutoStrategy> result = new Result<>();
        AiKuaishouAccountAutoStrategy aiKuaishouAccountAutoStrategy = aiKuaishouAccountAutoStrategyService.getById(id);
        if (aiKuaishouAccountAutoStrategy == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(aiKuaishouAccountAutoStrategy);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<AiKuaishouAccountAutoStrategy> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                AiKuaishouAccountAutoStrategy aiKuaishouAccountAutoStrategy = JSON.parseObject(deString, AiKuaishouAccountAutoStrategy.class);
                queryWrapper = QueryGenerator.initQueryWrapper(aiKuaishouAccountAutoStrategy, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<AiKuaishouAccountAutoStrategy> pageList = aiKuaishouAccountAutoStrategyService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "自动投放重构-账户策略表列表");
        mv.addObject(NormalExcelConstants.CLASS, AiKuaishouAccountAutoStrategy.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("自动投放重构-账户策略表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<AiKuaishouAccountAutoStrategy> listAiKuaishouAccountAutoStrategys = ExcelImportUtil.importExcel(file.getInputStream(), AiKuaishouAccountAutoStrategy.class, params);
                aiKuaishouAccountAutoStrategyService.saveBatch(listAiKuaishouAccountAutoStrategys);
                return Result.ok("文件导入成功！数据行数:" + listAiKuaishouAccountAutoStrategys.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
