package cn.com.ctop.kuaishou.modules.ai.controller;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.ai.entity.AiKuaishouAccountAutoTarget;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouAccountAutoTargetService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 自动投放重构-本地定向表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-10-12
 */
@Slf4j
@Api(tags = "自动投放重构-本地定向表")
@RestController
@RequestMapping("/auto/aiKuaishouAccountAutoTarget")
public class AiKuaishouAccountAutoTargetController {
    @Autowired
    private IAiKuaishouAccountAutoTargetService aiKuaishouAccountAutoTargetService;

    /**
     * 分页列表查询
     *
     * @param aiKuaishouAccountAutoTarget
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "自动投放重构-本地定向表-分页列表查询", notes = "自动投放重构-本地定向表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<Object> queryPageList(Long accountId, String targetName,Integer platformOs, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                        @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        try {
            if (Check.isNull(accountId)||Check.isNull(platformOs)) {
                return Result.error("缺少参数");
            }
            return aiKuaishouAccountAutoTargetService.queryPageList(accountId, targetName,platformOs, pageNo, pageSize);
        } catch (Exception e) {
            log.error("查询异常,", e.getMessage());
            e.printStackTrace();
        }
        return Result.error("查询失败");
    }

    /**
     * 创建定向包
     *
     * @param
     * @return
     */
    @PostMapping(value = "/addLocalTarget")
    public Result<Object> addLocalTarget(@RequestBody JSONObject requestJson) {
        try {
            if (Check.isNull(requestJson.getString("targetName"))) {
                return Result.error("定向包名称为必传项");
            }
            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                return Result.error("请选择账户id");
            }
            return aiKuaishouAccountAutoTargetService.addLocalTarget(requestJson);
        } catch (Exception e) {
            log.error("创建定向包失败", e);
            return Result.error("创建失败" + e.getMessage());
        }
    }

    /**
     * 修改定向包
     *
     * @param
     * @return
     */
    @PostMapping(value = "/updateLocalTarget")
    public Result<Object> updateLocalTarget(@RequestBody JSONObject requestJson) {
        try {
            if (Check.isNull(requestJson.getString("id"))) {
                return Result.error("请选择定向包");
            }
            AiKuaishouAccountAutoTarget aiKuaishouAccountAutoTarget = aiKuaishouAccountAutoTargetService.getById(requestJson.getString("id"));
            if (aiKuaishouAccountAutoTarget == null) {
                return Result.ok("修改失败，未查到对应定向包");
            }
            return aiKuaishouAccountAutoTargetService.updateLocalTarget(requestJson);
        } catch (Exception e) {
            return Result.error("修改异常");
        }
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放重构-本地定向表-通过id删除", notes = "自动投放重构-本地定向表-通过id删除")
    @GetMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            aiKuaishouAccountAutoTargetService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "自动投放重构-本地定向表-批量删除", notes = "自动投放重构-本地定向表-批量删除")
    @GetMapping(value = "/deleteBatch")
    public Result<AiKuaishouAccountAutoTarget> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<AiKuaishouAccountAutoTarget> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.aiKuaishouAccountAutoTargetService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放重构-本地定向表-通过id查询", notes = "自动投放重构-本地定向表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<Object> queryById(@RequestParam(name = "id", required = true) String id) {
        try {
            if (Check.isNull(id)) {
                return Result.error("请选择定向");
            }
            AiKuaishouAccountAutoTarget aiKuaishouAccountAutoTarget = aiKuaishouAccountAutoTargetService.getById(id);
            if (aiKuaishouAccountAutoTarget == null) {
                return Result.ok("未找到对应实体");
            } else {
                JSONObject data = JSONObject.parseObject(JSONObject.toJSONString(aiKuaishouAccountAutoTarget)).toJavaObject(JSONObject.class);
                if (!Check.isNull(data.getString("keyword"))) {
                    data.put("keyword", data.getJSONArray("keyword"));
                }
                return Result.ok(data);
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error("查询异常");
        }
    }


    @PostMapping(value = "/getAutoTargetByIds")
    public Result<Object> getAutoTargetByIds(@RequestBody JSONObject data) {
        try {
            JSONArray ids = data.getJSONArray("ids");
            if (Check.isNull(ids)) {
                return Result.error("缺少参数");
            }
            List<String> list = JSONObject.parseObject(JSONObject.toJSONString(ids), List.class);
            if (Check.isNull(list)) {
                return Result.error("缺少参数");
            }
            List<AiKuaishouAccountAutoTarget> targets = aiKuaishouAccountAutoTargetService.getAutoTargetByIds(list);
            return Result.ok(targets);
        } catch (Exception e) {
            log.error("查询异常,", e);
            return Result.error("查询失败，" + e.getMessage());
        }
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<AiKuaishouAccountAutoTarget> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                AiKuaishouAccountAutoTarget aiKuaishouAccountAutoTarget = JSON.parseObject(deString, AiKuaishouAccountAutoTarget.class);
                queryWrapper = QueryGenerator.initQueryWrapper(aiKuaishouAccountAutoTarget, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<AiKuaishouAccountAutoTarget> pageList = aiKuaishouAccountAutoTargetService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "自动投放重构-本地定向表列表");
        mv.addObject(NormalExcelConstants.CLASS, AiKuaishouAccountAutoTarget.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("自动投放重构-本地定向表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<AiKuaishouAccountAutoTarget> listAiKuaishouAccountAutoTargets = ExcelImportUtil.importExcel(file.getInputStream(), AiKuaishouAccountAutoTarget.class, params);
                aiKuaishouAccountAutoTargetService.saveBatch(listAiKuaishouAccountAutoTargets);
                return Result.ok("文件导入成功！数据行数:" + listAiKuaishouAccountAutoTargets.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
