package cn.com.ctop.kuaishou.modules.batch.controller;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouGroupTemplate;
import cn.com.ctop.kuaishou.modules.batch.entity.KuaiShouImageGet;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouGroupTemplateService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaiShouImageGetService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 创建组模板
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-12-15
 */
@Slf4j
@Api(tags = "创建组模板")
@RestController
@RequestMapping("/batch/kuaiShouGroupTemplate")
public class KuaiShouGroupTemplateController {
    @Autowired
    private IKuaiShouGroupTemplateService kuaiShouGroupTemplateService;
    @Autowired
    private ICtopOauthTokenService tokenService;
    static ExecutorService executorService = Executors.newFixedThreadPool(10);
    @Autowired
    private IKuaiShouImageGetService iKuaiShouImageGetService;

    @PostMapping(value = "/getLibraryConfig")
    public Result<JSONObject> getLibraryConfig(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();
        try {
            if (Check.isNull(requestJson)) {
                throw new Exception("参数不能为空");
            }
            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                throw new Exception("请传如账户ID");
            }
            JSONObject returnJson = kuaiShouGroupTemplateService.getLibraryConfig(accountId);
            result.setSuccess(true);
            result.setResult(returnJson);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }

    @PostMapping(value = "/productList")
    public JSONObject productList(@RequestBody JSONObject requestJson) {
        JSONObject returnJson = new JSONObject();
        try {
            if (Check.isNull(requestJson)) {
                throw new Exception("参数不能为空");
            }
            Long accountId = requestJson.getLong("accountId");
            if (Check.isNull(accountId)) {
                throw new Exception("请传如账户ID");
            }
            CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
            if (Check.isNull(token)) {
                throw new Exception("未获取到授权信息");
            }

            Long libraryId = requestJson.getLong("libraryId");
            if (Check.isNull(libraryId)) {
                throw new Exception("请传入商品ID");
            }
            Integer page = requestJson.getInteger("page");
            if (Check.isNull(page)) {
                throw new Exception("请传入商品请求页数");
            }
            Integer pageSize = requestJson.getInteger("pageSize");
            if (Check.isNull(pageSize)) {
                throw new Exception("请传入商品请求条数");
            }
            String libraryName = requestJson.getString("libraryName");
            returnJson = kuaiShouGroupTemplateService.productList(accountId, token.getAccessToken(), libraryId, page, pageSize, libraryName);
        } catch (Exception e) {
            e.printStackTrace();
            returnJson.put("code", -1);
            returnJson.put("message", e.getMessage());
        }
        return returnJson;
    }


    /**
     * 批量创建广告组
     *
     * @param requestJson
     * @return
     */
    @PostMapping(value = "/submitUnitTask")
    public Result<JSONObject> submitUnitTask(@RequestBody JSONObject requestJson) {
        Result<JSONObject> result = new Result<>();
        try {
            if (Check.isNull(requestJson)) {
                throw new Exception("参数不能为空");
            }
            JSONObject returnJson = kuaiShouGroupTemplateService.submitUnitTask(requestJson);
            result.setSuccess(true);
            result.setResult(returnJson);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }


    @GetMapping(value = "/createUnit")
    public Result<JSONObject> createUnit(Long id) {
        Result<JSONObject> result = new Result<>();
        try {
            KuaiShouGroupTemplate template = kuaiShouGroupTemplateService.getById(id);
            if (!Check.isNull(template)) {
                KuaiShouGroupTemplate updateTemplate = new KuaiShouGroupTemplate();
                updateTemplate.setId(template.getId());
                updateTemplate.setTaskStatus(1);
                kuaiShouGroupTemplateService.updateById(updateTemplate);   // 修改状态为创建中
                executorService.submit(() -> kuaiShouGroupTemplateService.createUnit(template));
            }
            result.setSuccess(true);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }

    @GetMapping(value = "/createUnitAgain")
    public Result<Object> createUnitAgain(Long id) {
        try {
            KuaiShouGroupTemplate template = kuaiShouGroupTemplateService.getById(id);
            if (!Check.isNull(template)) {
                KuaiShouGroupTemplate updateTemplate = new KuaiShouGroupTemplate();
                updateTemplate.setId(template.getId());
                updateTemplate.setTaskStatus(1);
                kuaiShouGroupTemplateService.updateById(updateTemplate);   // 修改状态为创建中
                executorService.submit(() -> kuaiShouGroupTemplateService.createUnitAgain(template));
            }
        } catch (Exception e) {
            e.printStackTrace();
            return Result.error(e.getMessage());
        }
        return Result.ok();
    }

    /**
     * 获取api今日创建创意数
     *
     * @param accountId
     * @return
     */
    @GetMapping(value = "/getApiCreateCount")
    public Result<Object> getApiCreateCount(Long accountId) {
        Result<Object> result = new Result<>();
        try {
            if (Check.isNull(accountId)) {
                throw new Exception("accountId参数不能为空");
            }
            String date = DateUtils.getNowDate("yyyy-MM-dd");
            return kuaiShouGroupTemplateService.getApiCreateCount(accountId, date);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }


    @GetMapping(value = "/getUrlByPhotoId")
    public Result<JSONObject> getUrlByPhotoId(Long photoId) {
        Result<JSONObject> result = new Result<>();
        try {
            if (Check.isNull(photoId)) {
                throw new Exception("photoId参数不能为空");
            }
            JSONObject urlJson = kuaiShouGroupTemplateService.getUrlByPhotoId(photoId);
            result.setSuccess(true);
            result.setResult(urlJson);
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage(e.getMessage());
        }
        return result;

    }

    @PostMapping(value = "/getUrlByPhotoIds")
    public Result<Object> getUrlByPhotoIds(@RequestBody JSONObject requestJson) {
        try {
            Long accountId = requestJson.getLong("accountId");
            JSONArray photoIds = requestJson.getJSONArray("photoIds");
            if (photoIds.isEmpty()) {
                throw new Exception("参数不能为空");
            }
            List<JSONObject> list = kuaiShouGroupTemplateService.getUrlByPhotoIds(accountId, photoIds);
            return Result.ok(list);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("查询失败");

    }


    @PostMapping(value = "/queryUrlByImageTokens")
    public Result<Object> queryUrlByImageTokens(@RequestBody JSONObject requestJson) {
        try {
            Long accountId = requestJson.getLong("accountId");
            JSONArray imageTokens = requestJson.getJSONArray("imageTokens");
            if (Check.isNull(accountId) || imageTokens.isEmpty()) {
                throw new Exception("参数不能为空");
            }
            List<KuaiShouImageGet> list = iKuaiShouImageGetService.getUrlByImageTokens(accountId, imageTokens);
            return Result.ok(list);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("查询失败");

    }


    /**
     * 分页列表查询
     *
     * @param kuaiShouGroupTemplate
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "创建组模板-分页列表查询")
    @ApiOperation(value = "创建组模板-分页列表查询", notes = "创建组模板-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaiShouGroupTemplate>> queryPageList(KuaiShouGroupTemplate kuaiShouGroupTemplate,
                                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                              @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                              HttpServletRequest req) {
        Result<IPage<KuaiShouGroupTemplate>> result = new Result<>();
        QueryWrapper<KuaiShouGroupTemplate> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouGroupTemplate, req.getParameterMap());
        queryWrapper.orderByDesc("create_time");
        Page<KuaiShouGroupTemplate> page = new Page<KuaiShouGroupTemplate>(pageNo, pageSize);
        IPage<KuaiShouGroupTemplate> pageList = kuaiShouGroupTemplateService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param kuaiShouGroupTemplate
     * @return
     */
    @AutoLog(value = "创建组模板-添加")
    @ApiOperation(value = "创建组模板-添加", notes = "创建组模板-添加")
    @PostMapping(value = "/add")
    public Result<KuaiShouGroupTemplate> add(@RequestBody KuaiShouGroupTemplate kuaiShouGroupTemplate) {
        Result<KuaiShouGroupTemplate> result = new Result<>();
        try {
            kuaiShouGroupTemplateService.save(kuaiShouGroupTemplate);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaiShouGroupTemplate
     * @return
     */
    @AutoLog(value = "创建组模板-编辑")
    @ApiOperation(value = "创建组模板-编辑", notes = "创建组模板-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaiShouGroupTemplate> edit(@RequestBody KuaiShouGroupTemplate kuaiShouGroupTemplate) {
        Result<KuaiShouGroupTemplate> result = new Result<KuaiShouGroupTemplate>();
        KuaiShouGroupTemplate kuaiShouGroupTemplateEntity = kuaiShouGroupTemplateService.getById(kuaiShouGroupTemplate.getId());
        if (kuaiShouGroupTemplateEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaiShouGroupTemplateService.updateById(kuaiShouGroupTemplate);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "创建组模板-通过id删除")
    @ApiOperation(value = "创建组模板-通过id删除", notes = "创建组模板-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaiShouGroupTemplateService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "创建组模板-批量删除")
    @ApiOperation(value = "创建组模板-批量删除", notes = "创建组模板-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouGroupTemplate> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouGroupTemplate> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaiShouGroupTemplateService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "创建组模板-通过id查询")
    @ApiOperation(value = "创建组模板-通过id查询", notes = "创建组模板-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaiShouGroupTemplate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouGroupTemplate> result = new Result<>();
        KuaiShouGroupTemplate kuaiShouGroupTemplate = kuaiShouGroupTemplateService.getById(id);
        if (kuaiShouGroupTemplate == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaiShouGroupTemplate);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaiShouGroupTemplate> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaiShouGroupTemplate kuaiShouGroupTemplate = JSON.parseObject(deString, KuaiShouGroupTemplate.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouGroupTemplate, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaiShouGroupTemplate> pageList = kuaiShouGroupTemplateService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "创建组模板列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaiShouGroupTemplate.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("创建组模板列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaiShouGroupTemplate> listKuaiShouGroupTemplates = ExcelImportUtil.importExcel(file.getInputStream(), KuaiShouGroupTemplate.class, params);
                kuaiShouGroupTemplateService.saveBatch(listKuaiShouGroupTemplates);
                return Result.ok("文件导入成功！数据行数:" + listKuaiShouGroupTemplates.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
