package cn.com.ctop.kuaishou.modules.clean.controller;


import cn.com.ctop.common.module.entity.Project;
import cn.com.ctop.common.module.service.IProjectService;
import cn.com.ctop.common.module.service.ISysUserService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.clean.entity.ProjectMonitorWhiteList;
import cn.com.ctop.kuaishou.modules.clean.service.IProjectMonitorWhiteListService;
import cn.com.ctop.kuaishou.modules.clean.service.ITrackMonitorConfigureService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.entity.SysUser;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 监测链接预警-项目白名单
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-09-23
 */
@Slf4j
@Api(tags = "监测链接预警-项目白名单")
@RestController
@RequestMapping("/monitor/projectMonitorWhiteList")
public class ProjectMonitorWhiteListController {
    @Autowired
    private IProjectMonitorWhiteListService projectMonitorWhiteListService;
    @Autowired
    private ISysUserService userService;
    @Autowired
    private IProjectService projectService;
    @Autowired
    private ITrackMonitorConfigureService configureService;


    @PostMapping(value = "/getProjectAndAccountIds")
    public Result<List<JSONObject>> getProjectAndAccountIds(@RequestBody JSONObject requestBody) {
        Result<List<JSONObject>> result = new Result<>();
        String userId = requestBody.getString("userId");
        List<JSONObject> list = projectMonitorWhiteListService.getProjectAndAccountIds(userId);
        result.setSuccess(true);
        result.setResult(list);
        return result;
    }


    /**
     * 分页列表查询
     *
     * @param projectMonitorWhiteList
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @ApiOperation(value = "监测链接预警-项目白名单-分页列表查询", notes = "监测链接预警-项目白名单-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ProjectMonitorWhiteList>> queryPageList(ProjectMonitorWhiteList projectMonitorWhiteList,
                                                                @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                HttpServletRequest req) {
        Result<IPage<ProjectMonitorWhiteList>> result = new Result<>();
        QueryWrapper<ProjectMonitorWhiteList> queryWrapper = QueryGenerator.initQueryWrapper(projectMonitorWhiteList, req.getParameterMap());
        Page<ProjectMonitorWhiteList> page = new Page<ProjectMonitorWhiteList>(pageNo, pageSize);
        IPage<ProjectMonitorWhiteList> pageList = projectMonitorWhiteListService.page(page, queryWrapper);
        List<ProjectMonitorWhiteList> records = pageList.getRecords();
        for (ProjectMonitorWhiteList whiteList : records) {
            String userId = whiteList.getUserId();
            Long projectId = whiteList.getProjectId();
            SysUser user = userService.getById(userId);
            if (!Check.isNull(user)) {
                whiteList.setUserName(user.getRealname());
            }
            Project project = projectService.getById(projectId);
            if (!Check.isNull(project)) {
                whiteList.setProjectName(project.getProjectName());

            }

        }


        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param projectMonitorWhiteList
     * @return
     */
    @ApiOperation(value = "监测链接预警-项目白名单-添加", notes = "监测链接预警-项目白名单-添加")
    @PostMapping(value = "/add")
    public Result<ProjectMonitorWhiteList> add(@RequestBody ProjectMonitorWhiteList projectMonitorWhiteList) {
        Result<ProjectMonitorWhiteList> result = new Result<>();
        try {
            Long projectId = projectMonitorWhiteList.getProjectId();
            QueryWrapper<ProjectMonitorWhiteList> monitorWhiteListQueryWrapper = new QueryWrapper<>();
            monitorWhiteListQueryWrapper.eq("project_id", projectId);
            monitorWhiteListQueryWrapper.last("limit 1");
            ProjectMonitorWhiteList one = projectMonitorWhiteListService.getOne(monitorWhiteListQueryWrapper);
            if (!Check.isNull(one)) {
                result.error("该项目已在白名单中");
                return result;
            }
            projectMonitorWhiteListService.save(projectMonitorWhiteList);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param projectMonitorWhiteList
     * @return
     */
    @ApiOperation(value = "监测链接预警-项目白名单-编辑", notes = "监测链接预警-项目白名单-编辑")
    @PutMapping(value = "/edit")
    public Result<ProjectMonitorWhiteList> edit(@RequestBody ProjectMonitorWhiteList projectMonitorWhiteList) {
        Result<ProjectMonitorWhiteList> result = new Result<ProjectMonitorWhiteList>();
        ProjectMonitorWhiteList projectMonitorWhiteListEntity = projectMonitorWhiteListService.getById(projectMonitorWhiteList.getId());
        if (projectMonitorWhiteListEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = projectMonitorWhiteListService.updateById(projectMonitorWhiteList);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "监测链接预警-项目白名单-通过id删除", notes = "监测链接预警-项目白名单-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            ProjectMonitorWhiteList byId = projectMonitorWhiteListService.getById(id);
            if (!Check.isNull(byId)) {
                Map<String, Object> deleteMap = new HashMap<>();
                deleteMap.put("project_id", byId.getProjectId());
                configureService.removeByMap(deleteMap);
                projectMonitorWhiteListService.removeById(id);
            }


        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "监测链接预警-项目白名单-批量删除", notes = "监测链接预警-项目白名单-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ProjectMonitorWhiteList> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<ProjectMonitorWhiteList> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.projectMonitorWhiteListService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "监测链接预警-项目白名单-通过id查询", notes = "监测链接预警-项目白名单-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ProjectMonitorWhiteList> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ProjectMonitorWhiteList> result = new Result<>();
        ProjectMonitorWhiteList projectMonitorWhiteList = projectMonitorWhiteListService.getById(id);
        if (projectMonitorWhiteList == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(projectMonitorWhiteList);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<ProjectMonitorWhiteList> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                ProjectMonitorWhiteList projectMonitorWhiteList = JSON.parseObject(deString, ProjectMonitorWhiteList.class);
                queryWrapper = QueryGenerator.initQueryWrapper(projectMonitorWhiteList, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<ProjectMonitorWhiteList> pageList = projectMonitorWhiteListService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "监测链接预警-项目白名单列表");
        mv.addObject(NormalExcelConstants.CLASS, ProjectMonitorWhiteList.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("监测链接预警-项目白名单列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<ProjectMonitorWhiteList> listProjectMonitorWhiteLists = ExcelImportUtil.importExcel(file.getInputStream(), ProjectMonitorWhiteList.class, params);
                projectMonitorWhiteListService.saveBatch(listProjectMonitorWhiteLists);
                return Result.ok("文件导入成功！数据行数:" + listProjectMonitorWhiteLists.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
