package cn.com.ctop.kuaishou.modules.clean.controller;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.clean.entity.TrackMonitorConfigure;
import cn.com.ctop.kuaishou.modules.clean.entity.TrackMonitorLogs;
import cn.com.ctop.kuaishou.modules.clean.service.ITrackMonitorConfigureService;
import cn.com.ctop.kuaishou.modules.clean.service.ITrackMonitorLogsService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 监测链接预警-日志
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2021-09-23
 */
@Slf4j
@Api(tags = "监测链接预警-日志")
@RestController
@RequestMapping("/monitor/trackMonitorLogs")
public class TrackMonitorLogsController {
    @Autowired
    private ITrackMonitorLogsService trackMonitorLogsService;
    @Autowired
    private IUserAllocationService userAllocationService;
    @Autowired
    private ICtopOauthTokenService tokenService;
    @Autowired
    private ITrackMonitorConfigureService trackMonitorConfigureService;
    static ExecutorService creativeExecutorService = Executors.newFixedThreadPool(3);
    static ExecutorService programCreativeExecutorService = Executors.newFixedThreadPool(3);

    /**
     * 自定义创意预警监测
     */

    @GetMapping(value = "/monitorCreative")
    public void monitorCreative(Long accountId) {
        UserAllocation userAllocation = userAllocationService.getByAccountId(accountId);
        if (Check.isNull(userAllocation)) {
            log.error("未获取到账户分配信息，accountId:{}", accountId);
            return;
        }
        if (userAllocation.getAccountStatus() == 1) {
            log.error("该账户已暂停投放，accountId:{}", accountId);
            return;
        }

        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        if (Check.isNull(token)) {
            log.error("该账户未获取到token信息，accountId:{}", accountId);
            return;
        }
        TrackMonitorConfigure configure = trackMonitorConfigureService.getByAccountId(accountId);
        if (Check.isNull(configure)) {
            log.error("该账户预警配置信息为空，accountId:{}", accountId);
            return;
        }

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        creativeExecutorService.submit(new Runnable() {
            @Override
            public void run() {
                trackMonitorLogsService.monitorCreative(configure, accountId, token.getAccessToken(), userAllocation.getUserId(), nowDate, nowDate, 1);
            }
        });
    }


    /**
     * 程序化创意预警监测
     *
     * @param accountId
     */
    @GetMapping(value = "/monitorProgramCreative")
    public void monitorProgramCreative(Long accountId) {
        UserAllocation userAllocation = userAllocationService.getByAccountId(accountId);
        if (Check.isNull(userAllocation)) {
            log.error("未获取到账户分配信息，accountId:{}", accountId);
            return;
        }
        if (userAllocation.getAccountStatus() == 1) {
            log.error("该账户已暂停投放，accountId:{}", accountId);
            return;
        }

        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        if (Check.isNull(token)) {
            log.error("该账户未获取到token信息，accountId:{}", accountId);
            return;
        }
        TrackMonitorConfigure configure = trackMonitorConfigureService.getByAccountId(accountId);
        if (Check.isNull(configure)) {
            log.error("该账户预警配置信息为空，accountId:{}", accountId);
            return;
        }

        String nowDate = DateUtils.getNowDate("yyyy-MM-dd");
        programCreativeExecutorService.submit(new Runnable() {
            @Override
            public void run() {
                trackMonitorLogsService.monitorProgramCreative(configure, accountId, token.getAccessToken(), userAllocation.getUserId(), nowDate, nowDate, 1);
            }
        });

    }


    @GetMapping(value = "/list")
    public Result<IPage<TrackMonitorLogs>> queryPageList(TrackMonitorLogs trackMonitorLogs,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<TrackMonitorLogs>> result = new Result<>();
        QueryWrapper<TrackMonitorLogs> queryWrapper = QueryGenerator.initQueryWrapper(trackMonitorLogs, req.getParameterMap());
        Page<TrackMonitorLogs> page = new Page<TrackMonitorLogs>(pageNo, pageSize);
        IPage<TrackMonitorLogs> pageList = trackMonitorLogsService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param trackMonitorLogs
     * @return
     */
    @ApiOperation(value = "监测链接预警-日志-添加", notes = "监测链接预警-日志-添加")
    @PostMapping(value = "/add")
    public Result<TrackMonitorLogs> add(@RequestBody TrackMonitorLogs trackMonitorLogs) {
        Result<TrackMonitorLogs> result = new Result<>();
        try {
            trackMonitorLogsService.save(trackMonitorLogs);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param trackMonitorLogs
     * @return
     */
    @ApiOperation(value = "监测链接预警-日志-编辑", notes = "监测链接预警-日志-编辑")
    @PutMapping(value = "/edit")
    public Result<TrackMonitorLogs> edit(@RequestBody TrackMonitorLogs trackMonitorLogs) {
        Result<TrackMonitorLogs> result = new Result<TrackMonitorLogs>();
        TrackMonitorLogs trackMonitorLogsEntity = trackMonitorLogsService.getById(trackMonitorLogs.getId());
        if (trackMonitorLogsEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = trackMonitorLogsService.updateById(trackMonitorLogs);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "监测链接预警-日志-通过id删除", notes = "监测链接预警-日志-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            trackMonitorLogsService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "监测链接预警-日志-批量删除", notes = "监测链接预警-日志-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<TrackMonitorLogs> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<TrackMonitorLogs> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.trackMonitorLogsService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "监测链接预警-日志-通过id查询", notes = "监测链接预警-日志-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<TrackMonitorLogs> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<TrackMonitorLogs> result = new Result<>();
        TrackMonitorLogs trackMonitorLogs = trackMonitorLogsService.getById(id);
        if (trackMonitorLogs == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(trackMonitorLogs);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<TrackMonitorLogs> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                TrackMonitorLogs trackMonitorLogs = JSON.parseObject(deString, TrackMonitorLogs.class);
                queryWrapper = QueryGenerator.initQueryWrapper(trackMonitorLogs, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<TrackMonitorLogs> pageList = trackMonitorLogsService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "监测链接预警-日志列表");
        mv.addObject(NormalExcelConstants.CLASS, TrackMonitorLogs.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("监测链接预警-日志列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<TrackMonitorLogs> listTrackMonitorLogss = ExcelImportUtil.importExcel(file.getInputStream(), TrackMonitorLogs.class, params);
                trackMonitorLogsService.saveBatch(listTrackMonitorLogss);
                return Result.ok("文件导入成功！数据行数:" + listTrackMonitorLogss.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
