package cn.com.ctop.kuaishou.modules.live.controller;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.live.entity.LiveDataTouch;
import cn.com.ctop.kuaishou.modules.live.service.ILiveDataImportRecordService;
import cn.com.ctop.kuaishou.modules.live.service.ILiveDataTouchService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;

/**
 * 直播数据——触达人群报表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2022-04-02
 */
@Slf4j
@Api(tags = "直播数据——触达人群报表")
@RestController
@RequestMapping("/liveData/touch")
public class LiveDataTouchController {
    @Autowired
    private ILiveDataTouchService liveDataTouchService;

    @Autowired
    private ILiveDataImportRecordService recordService;

    /**
     * 分页列表查询
     */
    @GetMapping(value = "/list")
    public Result<IPage<LiveDataTouch>> queryPageList(LiveDataTouch liveDataTouch,
                                                      @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                      @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                      HttpServletRequest req) {
        Result<IPage<LiveDataTouch>> result = new Result<>();
        QueryWrapper<LiveDataTouch> queryWrapper = QueryGenerator.initQueryWrapper(new LiveDataTouch(), req.getParameterMap());
        Page<LiveDataTouch> page = new Page<LiveDataTouch>(pageNo, pageSize);
        if (!Check.isNull(liveDataTouch.getAccountId())) {
            queryWrapper.eq("account_id", liveDataTouch.getAccountId());
        }
        if (!Check.isNull(liveDataTouch.getStartTime())) {
            queryWrapper.ge("stat_date", liveDataTouch.getStartTime());
        }

        if (!Check.isNull(liveDataTouch.getEndTime())) {
            queryWrapper.le("stat_date", liveDataTouch.getEndTime());
        }

        if (!Check.isNull(liveDataTouch.getFanStrat())) {
            queryWrapper.like("fan_strat", liveDataTouch.getFanStrat());
        }

        if (!Check.isNull(liveDataTouch.getCustomerStrat())) {
            queryWrapper.like("customer_strat", liveDataTouch.getCustomerStrat());
        }
        queryWrapper.eq("state", 1);
        queryWrapper.orderByDesc("create_time");
        IPage<LiveDataTouch> pageList = liveDataTouchService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }


    /**
     * 导入EXCEL批量创建
     */
    @PostMapping(value = "/importExcel")
    @ResponseBody
    public Result<Object> importExcel(HttpServletResponse response, String userId, Long accountId, @RequestParam("file") MultipartFile file) {
        response.setHeader("Access-Control-Allow-Origin", "*");
        try {
            if (Check.isNull(file)) {
                return Result.error("执行失败，请上传应用Excel文件");
            }
            if (Check.isNull(accountId) || Check.isNull(userId)) {
                return Result.error("缺少参数");
            }
            Result<Object> result = liveDataTouchService.importExcel(accountId, file);
            recordService.addImportRecord(accountId, 2, userId);
            return result;

        } catch (Exception e) {
            e.printStackTrace();
            return Result.error("导入EXCEL失败，" + e.getMessage());
        }
    }


    /**
     * 添加
     *
     * @param liveDataTouch
     * @return
     */
    @PostMapping(value = "/add")
    public Result<LiveDataTouch> add(@RequestBody LiveDataTouch liveDataTouch) {
        Result<LiveDataTouch> result = new Result<>();
        try {
            liveDataTouchService.save(liveDataTouch);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param liveDataTouch
     * @return
     */
    @ApiOperation(value = "直播数据——触达人群报表-编辑", notes = "直播数据——触达人群报表-编辑")
    @PutMapping(value = "/edit")
    public Result<LiveDataTouch> edit(@RequestBody LiveDataTouch liveDataTouch) {
        Result<LiveDataTouch> result = new Result<LiveDataTouch>();
        LiveDataTouch liveDataTouchEntity = liveDataTouchService.getById(liveDataTouch.getId());
        if (liveDataTouchEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = liveDataTouchService.updateById(liveDataTouch);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "直播数据——触达人群报表-通过id删除", notes = "直播数据——触达人群报表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            liveDataTouchService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "直播数据——触达人群报表-批量删除", notes = "直播数据——触达人群报表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<LiveDataTouch> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<LiveDataTouch> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.liveDataTouchService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "直播数据——触达人群报表-通过id查询", notes = "直播数据——触达人群报表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<LiveDataTouch> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<LiveDataTouch> result = new Result<>();
        LiveDataTouch liveDataTouch = liveDataTouchService.getById(id);
        if (liveDataTouch == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(liveDataTouch);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<LiveDataTouch> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                LiveDataTouch liveDataTouch = JSON.parseObject(deString, LiveDataTouch.class);
                queryWrapper = QueryGenerator.initQueryWrapper(liveDataTouch, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<LiveDataTouch> pageList = liveDataTouchService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "直播数据——触达人群报表列表");
        mv.addObject(NormalExcelConstants.CLASS, LiveDataTouch.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("直播数据——触达人群报表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }
}
