package cn.com.ctop.kuaishou.modules.report.controller;

import cn.com.ctop.common.module.entity.CtopOauthToken;
import cn.com.ctop.common.module.service.ICtopOauthTokenService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.ai.entity.AiKuaishouUnitLevelOperationRecord;
import cn.com.ctop.kuaishou.modules.ai.mapper.AiKuaishouUnitLevelOperationRecordMapper;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAudienceReportDaily;
import cn.com.ctop.kuaishou.modules.report.entity.KuaishouAudienceReportDailyAccount;
import cn.com.ctop.kuaishou.modules.report.service.IKuaishouAudienceReportDailyAccountService;
import cn.com.ctop.kuaishou.modules.report.service.IKuaishouAudienceReportDailyService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 人群分析报表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2020-12-03
 */
@Slf4j
@Api(tags = "人群分析报表")
@RestController
@RequestMapping("/audienceReport")
public class KuaishouAudienceReportDailyController {
    @Autowired
    private IKuaishouAudienceReportDailyService kuaishouAudienceReportDailyService;
    @Autowired
    private IKuaishouAudienceReportDailyAccountService kuaishouAudienceReportDailyAccountService;

    /**
     * 分页列表查询
     *
     * @param kuaishouAudienceReportDaily
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "人群分析报表-分页列表查询")
    @ApiOperation(value = "人群分析报表-分页列表查询", notes = "人群分析报表-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<KuaishouAudienceReportDaily>> queryPageList(KuaishouAudienceReportDaily kuaishouAudienceReportDaily,
                                                                    @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                    @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                    HttpServletRequest req) {
        Result<IPage<KuaishouAudienceReportDaily>> result = new Result<>();
        QueryWrapper<KuaishouAudienceReportDaily> queryWrapper = QueryGenerator.initQueryWrapper(kuaishouAudienceReportDaily, req.getParameterMap());
        Page<KuaishouAudienceReportDaily> page = new Page<KuaishouAudienceReportDaily>(pageNo, pageSize);
        IPage<KuaishouAudienceReportDaily> pageList = kuaishouAudienceReportDailyService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 分页列表查询
     *
     * @param kuaishouAudienceReportDaily
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "人群分析报表-分页列表查询")
    @ApiOperation(value = "人群分析报表-分页列表查询", notes = "人群分析报表-分页列表查询")
    @GetMapping(value = "/queryPageList")
    public PageInfo<JSONObject> queryPageLists(KuaishouAudienceReportDailyAccount audienceReport, @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo, @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize) {
        return kuaishouAudienceReportDailyAccountService.queryPageLists(audienceReport, pageNo, pageSize);
    }

    /**
     * 首页报表数据接口
     *
     * @param
     * @return org.jeecg.common.api.vo.Result<java.lang.Object>
     * @throws
     * @author ZHAOXA
     */
    @AutoLog(value = "首页报表数据接口")
    @ApiOperation(value = "首页报表数据接口", notes = "首页报表数据接口")
    @GetMapping(value = "/queryAllReportForm")
    public Result<Object> queryAllReportForm(KuaishouAudienceReportDailyAccount audienceReport) {
        try {
            return kuaishouAudienceReportDailyAccountService.queryAllReportForm(audienceReport);
        } catch (Exception e) {
            log.error("首页报表数据接口", e);
        }
        return Result.error("首页报表数据接口失败");
    }

    /**
     * 添加
     *
     * @param kuaishouAudienceReportDaily
     * @return
     */
    @AutoLog(value = "人群分析报表-添加")
    @ApiOperation(value = "人群分析报表-添加", notes = "人群分析报表-添加")
    @PostMapping(value = "/add")
    public Result<KuaishouAudienceReportDaily> add(@RequestBody KuaishouAudienceReportDaily kuaishouAudienceReportDaily) {
        Result<KuaishouAudienceReportDaily> result = new Result<>();
        try {
            kuaishouAudienceReportDailyService.save(kuaishouAudienceReportDaily);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param kuaishouAudienceReportDaily
     * @return
     */
    @AutoLog(value = "人群分析报表-编辑")
    @ApiOperation(value = "人群分析报表-编辑", notes = "人群分析报表-编辑")
    @PutMapping(value = "/edit")
    public Result<KuaishouAudienceReportDaily> edit(@RequestBody KuaishouAudienceReportDaily kuaishouAudienceReportDaily) {
        Result<KuaishouAudienceReportDaily> result = new Result<KuaishouAudienceReportDaily>();
        KuaishouAudienceReportDaily kuaishouAudienceReportDailyEntity = kuaishouAudienceReportDailyService.getById(kuaishouAudienceReportDaily.getId());
        if (kuaishouAudienceReportDailyEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = kuaishouAudienceReportDailyService.updateById(kuaishouAudienceReportDaily);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "人群分析报表-通过id删除")
    @ApiOperation(value = "人群分析报表-通过id删除", notes = "人群分析报表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            kuaishouAudienceReportDailyService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "人群分析报表-批量删除")
    @ApiOperation(value = "人群分析报表-批量删除", notes = "人群分析报表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<KuaishouAudienceReportDaily> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaishouAudienceReportDaily> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.kuaishouAudienceReportDailyService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "人群分析报表-通过id查询")
    @ApiOperation(value = "人群分析报表-通过id查询", notes = "人群分析报表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<KuaishouAudienceReportDaily> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaishouAudienceReportDaily> result = new Result<>();
        KuaishouAudienceReportDaily kuaishouAudienceReportDaily = kuaishouAudienceReportDailyService.getById(id);
        if (kuaishouAudienceReportDaily == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(kuaishouAudienceReportDaily);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<KuaishouAudienceReportDaily> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                KuaishouAudienceReportDaily kuaishouAudienceReportDaily = JSON.parseObject(deString, KuaishouAudienceReportDaily.class);
                queryWrapper = QueryGenerator.initQueryWrapper(kuaishouAudienceReportDaily, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<KuaishouAudienceReportDaily> pageList = kuaishouAudienceReportDailyService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "人群分析报表列表");
        mv.addObject(NormalExcelConstants.CLASS, KuaishouAudienceReportDaily.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("人群分析报表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<KuaishouAudienceReportDaily> listKuaishouAudienceReportDailys = ExcelImportUtil.importExcel(file.getInputStream(), KuaishouAudienceReportDaily.class, params);
                kuaishouAudienceReportDailyService.saveBatch(listKuaishouAudienceReportDailys);
                return Result.ok("文件导入成功！数据行数:" + listKuaishouAudienceReportDailys.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

    static ExecutorService executorService = Executors.newFixedThreadPool(10);
    @Autowired
    private ICtopOauthTokenService tokenService;
    static CountDownLatch countDownLatch = null;
    @Resource
    AiKuaishouUnitLevelOperationRecordMapper kuaishouUnitLevelOperationRecordMapper;

    @GetMapping(value = "/getAudienceDataNew")
    public void getAudienceData(@RequestParam(required = false, defaultValue = "2019-01-01") String date) throws Exception {
        int page = 1;
        int pageSize = 2000;
        if (date.equals("2019-01-01")) {
            date = DateUtils.formatDate(DateUtils.addDay(DateUtils.getNowDate(), -1), "yyyy-MM-dd");
        }
        int count = 0;
        List<CtopOauthToken> tokens = tokenService.getByProjectId(458L);    //淘特项目
        SimpleDateFormat dsdf = new SimpleDateFormat("yyyy-MM-dd");
        for (CtopOauthToken token : tokens) {
            Date enDate = DateUtils.addDay(DateUtils.getNowDate(), -1);
            if (enDate.after(token.getAccessTokenExpiresIn())) {   //过期跳过
                continue;
            }
            Date startDate = DateUtils.addDay(enDate, -30);
            log.info("accountId=" + token.getAccountId() + "开始：" + DateUtils.getLastDay(date, 30) + "结束：" + DateUtils.getLastDay(date, 1));
            List<AiKuaishouUnitLevelOperationRecord> planIdsAndUnitIds = kuaishouUnitLevelOperationRecordMapper.selectAllByAccountIdAndDate(token.getAccountId(), startDate, enDate);
            if (Check.isNull(planIdsAndUnitIds)) {
                continue;
            }
            log.info("账户总数:" + tokens.size() + "当前：" + ++count);
            countDownLatch = new CountDownLatch(planIdsAndUnitIds.size());
            String finalDate = date;
            planIdsAndUnitIds.forEach(planIdAndUnitId -> executorService.submit(() -> {
                try {
                    // province: 省级地域分布, city: 地级地域分布,gender: 性别分布,ageSegment: 年龄分布clientId: 系统分布 businessInterestTags: 商业兴趣分布
                    kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "province", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                    kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "city", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                    kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "gender", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                    kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "clientId", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                    kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "ageSegment", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                    kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "businessInterestTags", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                } catch (Exception e) {
                } finally {
                    countDownLatch.countDown();
                }
            }));
            countDownLatch.await();
        }
    }


    @GetMapping(value = "/getOneAudienceDataNew")
    public void getOneAudienceDataNew(Long accountId) throws Exception {
        int page = 1;
        int pageSize = 2000;
        int count = 0;
        CtopOauthToken token = tokenService.getTokenByAccountId(accountId);
        Date enDate = DateUtils.addDay(DateUtils.getNowDate(), -1);
        if (enDate.after(token.getAccessTokenExpiresIn())) {   //过期跳过
            return;
        }
        Date startDate = DateUtils.addDay(enDate, -30);
        List<AiKuaishouUnitLevelOperationRecord> planIdsAndUnitIds = kuaishouUnitLevelOperationRecordMapper.selectAllByAccountIdAndDate(token.getAccountId(), startDate, enDate);
        if (Check.isNull(planIdsAndUnitIds)) {
            return;
        }
        countDownLatch = new CountDownLatch(planIdsAndUnitIds.size());
        String finalDate = DateUtils.formatDate(enDate, "yyyy-MM-dd");;
        planIdsAndUnitIds.forEach(planIdAndUnitId -> executorService.submit(() -> {
            try {
                // province: 省级地域分布, city: 地级地域分布,gender: 性别分布,ageSegment: 年龄分布clientId: 系统分布 businessInterestTags: 商业兴趣分布
                kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "province", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "city", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "gender", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "clientId", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
                kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "ageSegment", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
//                kuaishouAudienceReportDailyService.crowdAnalysisReport2(token, finalDate, "businessInterestTags", planIdAndUnitId.getCampaignId(), planIdAndUnitId.getUnitId(), page, pageSize);
            } catch (Exception e) {
            } finally {
                countDownLatch.countDown();
            }
        }));
    }

}
