package cn.com.ctop.kuaishou.modules.v3.controller;

import cn.com.ctop.common.module.constant.CtopRoleCodeConstant;
import cn.com.ctop.common.module.entity.UserAllocation;
import cn.com.ctop.common.module.service.IUserAllocationService;
import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.kuaishou.modules.ai.service.IAiKuaishouAdvertiserStrategyService;
import cn.com.ctop.kuaishou.modules.batch.service.IKuaishouVideoRelateCreativesService;
import cn.com.ctop.kuaishou.modules.document.service.DocumentLibraryService;
import cn.com.ctop.kuaishou.modules.v3.entity.V3KuaishouAccountAutoStrategy;
import cn.com.ctop.kuaishou.modules.v3.service.V3KuaishouAccountAutoStrategyService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.DateUtils;
import org.jeecg.common.util.RedisUtil;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.system.service.ISysRoleService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 自动投放v3-账户策略表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2022-10-24
 */
@Slf4j
@Api(tags = "自动投放v3-账户策略表")
@RestController
@RequestMapping("/auto/v3KuaishouAccountAutoStrategy")
public class V3KuaishouAccountAutoStrategyController {
    @Autowired
    private V3KuaishouAccountAutoStrategyService v3KuaishouAccountAutoStrategyService;

    @Autowired
    private ISysRoleService sysRoleService;

    @Autowired
    private IUserAllocationService allocationService;

    @Autowired
    private IKuaishouVideoRelateCreativesService kuaishouVideoRelateCreativesService;

    @Autowired
    private DocumentLibraryService documentLibraryService;

    @Autowired
    private IAiKuaishouAdvertiserStrategyService aiKuaishouAdvertiserStrategyService;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 分页列表查询
     *
     * @param V3KuaishouAccountAutoStrategy
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<PageInfo<V3KuaishouAccountAutoStrategy>> queryPageList(V3KuaishouAccountAutoStrategy strategy,
                                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                         @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                         HttpServletRequest req) {
        Result<PageInfo<V3KuaishouAccountAutoStrategy>> result = new Result<>();
        if (Check.isNull(strategy.getUserId())) {
            result.setSuccess(false);
            result.setMessage("请选择用户");
            return result;
        }
        String userId = strategy.getUserId();
        String roleCode = sysRoleService.getRoleCodeByUserId(userId);
        List<String> userIds = new ArrayList<>();
        if ("CompanyManager".equals(roleCode)) {
            userIds = sysRoleService.getCompanyUsers(roleCode, userId);
            strategy.setUserIds(userIds);
        } else if ("bytedanceDirector".equals(roleCode)) {
            userIds = sysRoleService.getCompanyUsers(roleCode, userId);
            strategy.setUserIds(userIds);
        } else if ("kuaishouDirector".equals(roleCode)) {
            userIds = sysRoleService.getCompanyUsers(roleCode, userId);
            strategy.setUserIds(userIds);
        } else if (CtopRoleCodeConstant.COMMON_ROLE_CODE_ADMIN.equals(roleCode)) {
            strategy.setUserIds(null);
        } else {
            userIds.add(userId);
            strategy.setUserIds(userIds);
        }
        PageHelper.startPage(pageNo, pageSize);
        PageInfo<V3KuaishouAccountAutoStrategy> pageInfo = v3KuaishouAccountAutoStrategyService.queryList(strategy);
        List<V3KuaishouAccountAutoStrategy> list = pageInfo.getList();
        List<V3KuaishouAccountAutoStrategy> setData = new ArrayList<>();
        String date = DateUtils.formatDate(new Date());
        if (null != list && !list.isEmpty()) {
            list.forEach(record -> {
                Long accountId = record.getAccountId();
                UserAllocation allocation = allocationService.getByAccountId(accountId);
                if (null != allocation) {
                    record.setAuthName(allocation.getAuthName());
                }
                String customkey = date + "_v3_custom_cnt_" + accountId;
                Integer customUnitCnt = (Integer) redisUtil.get(customkey);
                customUnitCnt = Check.isNull(customUnitCnt) ? 0 : customUnitCnt;
                record.setCustomCreatedUnitCnt(customUnitCnt);
                String programkey = date + "_v3_program_cnt_" + accountId;
                Integer programUnitCnt = (Integer) redisUtil.get(programkey);
                programUnitCnt = Check.isNull(programUnitCnt) ? 0 : programUnitCnt;
                record.setProgramCreatedUnitCnt(programUnitCnt);
            });
        }
        pageInfo.setList(list);
        result.setSuccess(true);
        result.setResult(pageInfo);
        return result;
    }

    /**
     * 添加
     *
     * @param V3KuaishouAccountAutoStrategy
     * @return
     */
    @ApiOperation(value = "自动投放重构-账户策略表-添加", notes = "自动投放重构-账户策略表-添加")
    @PostMapping(value = "/add")
    public Result<V3KuaishouAccountAutoStrategy> add(@RequestBody V3KuaishouAccountAutoStrategy autoStrategy) {
        Result<V3KuaishouAccountAutoStrategy> result = new Result<>();
        try {
            Long accountId = autoStrategy.getAccountId();
            QueryWrapper<V3KuaishouAccountAutoStrategy> queryWrapper = new QueryWrapper();
            queryWrapper.eq("account_id", accountId).last("limit 1");
            V3KuaishouAccountAutoStrategy one = v3KuaishouAccountAutoStrategyService.getOne(queryWrapper);
            if (null != one) {
                result.error500("该账户配置信息已经被创建");
                return result;
            }
            UserAllocation allocation = allocationService.getByAccountId(accountId);
            if (null != allocation) {
                autoStrategy.setUserId(allocation.getUserId());
            }
            Integer singleAppId = autoStrategy.getSingleAppid();
            if (null != singleAppId && singleAppId == 1) {
                if (autoStrategy.getCampaignType() == 2 || autoStrategy.getCampaignType() == 7) {
                    Long appId = Long.valueOf(JSONArray.parseArray(autoStrategy.getAppIdArray()).getString(0));
                    autoStrategy.setAppId(appId);
                }
            }
            v3KuaishouAccountAutoStrategyService.save(autoStrategy);
            //添加广告语到文案库
            documentLibraryService.insertBatch(autoStrategy.getDescription(), accountId);
            Thread thread = new Thread() {
                @Override
                public void run() {
                    kuaishouVideoRelateCreativesService.videoRelateCreatives(autoStrategy.getAccountId());
                }
            };
            thread.start();
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }


    /**
     * 编辑
     *
     * @param v3KuaishouAccountAutoStrategy
     * @return
     */
    @ApiOperation(value = "自动投放v3-账户策略表-编辑", notes = "自动投放v3-账户策略表-编辑")
    @PostMapping(value = "/edit")
    public Result<V3KuaishouAccountAutoStrategy> edit(@RequestBody V3KuaishouAccountAutoStrategy v3KuaishouAccountAutoStrategy) {
        Result<V3KuaishouAccountAutoStrategy> result = new Result<V3KuaishouAccountAutoStrategy>();
        V3KuaishouAccountAutoStrategy v3KuaishouAccountAutoStrategyEntity = v3KuaishouAccountAutoStrategyService.getById(v3KuaishouAccountAutoStrategy.getId());
        if (v3KuaishouAccountAutoStrategyEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = v3KuaishouAccountAutoStrategyService.updateById(v3KuaishouAccountAutoStrategy);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放v3-账户策略表-通过id删除", notes = "自动投放v3-账户策略表-通过id删除")
    @GetMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            v3KuaishouAccountAutoStrategyService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "自动投放v3-账户策略表-批量删除", notes = "自动投放v3-账户策略表-批量删除")
    @GetMapping(value = "/deleteBatch")
    public Result<V3KuaishouAccountAutoStrategy> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<V3KuaishouAccountAutoStrategy> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.v3KuaishouAccountAutoStrategyService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "自动投放v3-账户策略表-通过id查询", notes = "自动投放v3-账户策略表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<V3KuaishouAccountAutoStrategy> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<V3KuaishouAccountAutoStrategy> result = new Result<>();
        V3KuaishouAccountAutoStrategy v3KuaishouAccountAutoStrategy = v3KuaishouAccountAutoStrategyService.getById(id);
        if (v3KuaishouAccountAutoStrategy == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(v3KuaishouAccountAutoStrategy);
            result.setSuccess(true);
        }
        return result;
    }

}
