package cn.com.ctop.reimburse.mondule.controller;

import cn.com.ctop.reimburse.mondule.entity.ReimburseDict;
import cn.com.ctop.reimburse.mondule.service.IReimburseDictService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 报销申请字典表
 *
 * @author jeecg-boot
 * @version V1.0
 * @date 2022-11-21
 */
@Slf4j
@Api(tags = "报销申请字典表")
@RestController
@RequestMapping("/reimburse/dict")
public class ReimburseDictController {
    @Autowired
    private IReimburseDictService reimburseDictService;

    /**
     * 分页列表查询
     */
    @GetMapping(value = "/list")
    public Result<Object> queryPageList(ReimburseDict reimburseDict) {
        List<ReimburseDict> list = reimburseDictService.list();
        return Result.ok(list);
    }

    /**
     * 添加
     *
     * @param reimburseDict
     * @return
     */
    @ApiOperation(value = "报销申请字典表-添加", notes = "报销申请字典表-添加")
    @PostMapping(value = "/add")
    public Result<ReimburseDict> add(@RequestBody ReimburseDict reimburseDict) {
        Result<ReimburseDict> result = new Result<>();
        try {
            reimburseDictService.save(reimburseDict);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param reimburseDict
     * @return
     */
    @ApiOperation(value = "报销申请字典表-编辑", notes = "报销申请字典表-编辑")
    @PutMapping(value = "/edit")
    public Result<ReimburseDict> edit(@RequestBody ReimburseDict reimburseDict) {
        Result<ReimburseDict> result = new Result<ReimburseDict>();
        ReimburseDict reimburseDictEntity = reimburseDictService.getById(reimburseDict.getId());
        if (reimburseDictEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = reimburseDictService.updateById(reimburseDict);
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "报销申请字典表-通过id删除", notes = "报销申请字典表-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<?> delete(@RequestParam(name = "id") String id) {
        try {
            reimburseDictService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @ApiOperation(value = "报销申请字典表-批量删除", notes = "报销申请字典表-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ReimburseDict> deleteBatch(@RequestParam(name = "ids") String ids) {
        Result<ReimburseDict> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.reimburseDictService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "报销申请字典表-通过id查询", notes = "报销申请字典表-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ReimburseDict> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ReimburseDict> result = new Result<>();
        ReimburseDict reimburseDict = reimburseDictService.getById(id);
        if (reimburseDict == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(reimburseDict);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<ReimburseDict> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                ReimburseDict reimburseDict = JSON.parseObject(deString, ReimburseDict.class);
                queryWrapper = QueryGenerator.initQueryWrapper(reimburseDict, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<ReimburseDict> pageList = reimburseDictService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "报销申请字典表列表");
        mv.addObject(NormalExcelConstants.CLASS, ReimburseDict.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("报销申请字典表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<ReimburseDict> listReimburseDicts = ExcelImportUtil.importExcel(file.getInputStream(), ReimburseDict.class, params);
                reimburseDictService.saveBatch(listReimburseDicts);
                return Result.ok("文件导入成功！数据行数:" + listReimburseDicts.size());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                return Result.error("文件导入失败:" + e.getMessage());
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
