package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.utils.Check;
import cn.com.ctop.toutiao.modules.material.entity.ByteDanceVideoInfo;
import cn.com.ctop.toutiao.modules.material.service.IByteDanceVideoInfoService;
import cn.com.ctop.toutiao.modules.tool.callback.AuthTokenUtil;
import cn.com.ctop.toutiao.modules.tool.callback.BaseResponse;
import cn.com.ctop.toutiao.modules.tool.callback.CallbackResponse;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * @Description: 今日头条视频素材信息
 * @Author: jeecg-boot
 * @Date: 2019-07-29
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "今日头条视频素材信息")
@RestController
@RequestMapping("/ctop/byteDanceVideoInfo")
public class ByteDanceVideoInfoController {
    @Autowired
    private IByteDanceVideoInfoService byteDanceVideoInfoService;

    /**
     * 分页列表查询
     *
     * @param byteDanceVideoInfo
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-分页列表查询")
    @ApiOperation(value = "今日头条视频素材信息-分页列表查询", notes = "今日头条视频素材信息-分页列表查询")
    @GetMapping(value = "/list")
    public Result<IPage<ByteDanceVideoInfo>> queryPageList(ByteDanceVideoInfo byteDanceVideoInfo,
                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                           HttpServletRequest req) {
        Result<IPage<ByteDanceVideoInfo>> result = new Result<>();
        QueryWrapper<ByteDanceVideoInfo> queryWrapper = QueryGenerator.initQueryWrapper(byteDanceVideoInfo, req.getParameterMap());
        Page<ByteDanceVideoInfo> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceVideoInfo> pageList = byteDanceVideoInfoService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param byteDanceVideoInfo
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-添加")
    @ApiOperation(value = "今日头条视频素材信息-添加", notes = "今日头条视频素材信息-添加")
    @PostMapping(value = "/add")
    public Result<ByteDanceVideoInfo> add(@RequestBody ByteDanceVideoInfo byteDanceVideoInfo) {
        Result<ByteDanceVideoInfo> result = new Result<>();
        try {
            byteDanceVideoInfoService.save(byteDanceVideoInfo);
            result.success("添加成功！");
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param byteDanceVideoInfo
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-编辑")
    @ApiOperation(value = "今日头条视频素材信息-编辑", notes = "今日头条视频素材信息-编辑")
    @PutMapping(value = "/edit")
    public Result<ByteDanceVideoInfo> edit(@RequestBody ByteDanceVideoInfo byteDanceVideoInfo) {
        Result<ByteDanceVideoInfo> result = new Result<>();
        ByteDanceVideoInfo byteDanceVideoInfoEntity = byteDanceVideoInfoService.getById(byteDanceVideoInfo.getId());
        if (byteDanceVideoInfoEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = byteDanceVideoInfoService.updateById(byteDanceVideoInfo);
            if (ok) {
                result.success("修改成功!");
            } else {
                result.error500("未知错误");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-通过id删除")
    @ApiOperation(value = "今日头条视频素材信息-通过id删除", notes = "今日头条视频素材信息-通过id删除")
    @DeleteMapping(value = "/delete")
    public Result<Object> delete(@RequestParam(name = "id", required = true) String id) {
        try {
            byteDanceVideoInfoService.removeById(id);
        } catch (Exception e) {
            log.error("删除失败", e.getMessage());
            return Result.error("删除失败!");
        }
        return Result.ok("删除成功!");
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-批量删除")
    @ApiOperation(value = "今日头条视频素材信息-批量删除", notes = "今日头条视频素材信息-批量删除")
    @DeleteMapping(value = "/deleteBatch")
    public Result<ByteDanceVideoInfo> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<ByteDanceVideoInfo> result = new Result<>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.byteDanceVideoInfoService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @AutoLog(value = "今日头条视频素材信息-通过id查询")
    @ApiOperation(value = "今日头条视频素材信息-通过id查询", notes = "今日头条视频素材信息-通过id查询")
    @GetMapping(value = "/queryById")
    public Result<ByteDanceVideoInfo> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<ByteDanceVideoInfo> result = new Result<>();
        ByteDanceVideoInfo byteDanceVideoInfo = byteDanceVideoInfoService.getById(id);
        if (byteDanceVideoInfo == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(byteDanceVideoInfo);
            result.setSuccess(true);
        }
        return result;
    }


    /**
     * 广告违规素材推送
     */
    @RequestMapping(value = "/illegalVideosPush", method = {RequestMethod.GET, RequestMethod.POST})
    public CallbackResponse illegalVideosPush(HttpServletRequest request,
                                              @RequestParam(value = "challenge", defaultValue = "0") int challenge,
                                              @RequestParam(value = "event", defaultValue = "") String event) throws IOException {
        try {
            log.info("广告违规素材推送--------------------------challenge= " + challenge + "；event= " + event);
            // 处理 verify 事件
            if (null != event && event.equals("verify_webhook")) {
                return new CallbackResponse(BaseResponse.ok(), challenge);
            }

            // 订阅任务配置页面的 "验证密钥"
            String secretKey = "156a5d76ad354b43865ef57e5e416ca7";
            // 数据接收，验证消息
            AuthTokenUtil.InputStreamCacher cacher = new AuthTokenUtil.InputStreamCacher(request.getInputStream());
            boolean isValidToken = AuthTokenUtil.isValidToken(secretKey, cacher, request.getHeader("X-Open-Signature"));
            log.info("isValidToken=" + isValidToken);
            if (!isValidToken) {
                return new CallbackResponse(new BaseResponse(400, "invalid token"), 0);
            }
            JSONObject result = (JSONObject) JSONObject.parse(new String(AuthTokenUtil.readAsBytes(cacher)));
            log.info("转译结果：" + result);
            if (!Check.isNull(result)) {
                JSONObject data = result.getJSONObject("data");
                if (!Check.isNull(data)) {
                    JSONObject content = data.getJSONObject("content");
                    if (!Check.isNull(content)) {
                        Long agentId = content.getLong("agent_id");
                        JSONArray illegalMaterialIds = content.getJSONArray("illegal_material_ids");//违规素材集合
                        List<Long> list = JSONArray.parseArray(illegalMaterialIds.toJSONString(), Long.class);
                        Long promotionId = content.getLong("promotion_id");//计划id
                        byteDanceVideoInfoService.getIllegalVideosPushInfo(agentId, list, promotionId);
                    } else {
                        log.error("content数据为空");
                    }
                } else {
                    log.error("data数据为空");
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        // 数据处理流程...
        return new CallbackResponse(BaseResponse.ok(), 0);
    }


    /**
     * 获取广告违规素材详情
     */
    @RequestMapping(value = "/getIllegalVideosPushInfo")
    public void getIllegalVideosPushInfo(Long agentId, @RequestBody List<Long> list, Long promotionId) {
        byteDanceVideoInfoService.getIllegalVideosPushInfo(agentId, list, promotionId);
    }

}
